"""
CSV/Excel analysis page for the Z_n analyzer dashboard.
"""

import streamlit as st
import pandas as pd
import tempfile
import os
from lds_pipelines.visualization.plotly_charts import Plotter
from lds_pipelines.core.csv_processor import calculate_zn_values
from lds_pipelines.core.api_processor import check_alarm
from datetime import datetime, timedelta

def show_csv_page():
    st.title("📁 CSV/Excel Data Analysis")

    st.markdown("""
    Upload a CSV or Excel file containing your static data to analyze time-based metrics.

    The file should have the following columns:
    - `timestamps`: Date and time of the measurement is ISO format
    - `inlet_volume`, `inlet_pressure`, `outlet_volume`, `outlet_pressure`.
    """)

    uploaded_file = st.file_uploader("Choose a file", type=["csv", "xls", "xlsx"])

    if uploaded_file is not None:
        try:
            with tempfile.NamedTemporaryFile(delete=False, suffix=os.path.splitext(uploaded_file.name)[-1]) as temp_file:
                temp_file.write(uploaded_file.getvalue())
                temp_path = temp_file.name

            if uploaded_file.name.endswith(".csv"):
                df = pd.read_csv(temp_path)
            else:
                df = pd.read_excel(temp_path)

            os.unlink(temp_path)

            df['timestamps'] = pd.to_datetime(df['timestamps'])
            df = df.sort_values('timestamps').reset_index(drop=True)
            df= calculate_zn_values(df)

            leak_time= check_alarm(df)
            if check_alarm(df):
                if len(df)>300:
                    timestamp= leak_time- timedelta(hours=1)
                else:
                    timestamp= leak_time- timedelta(minutes=10)
                st.warning(f'Leak has been detected in the data at: {str(timestamp)}', icon="⚠️")


            y_column= "min_zn"

            plotter = Plotter()
            fig = plotter.create_plot(df, y_column)

            st.subheader("📈 Leak Likelihood")
            st.plotly_chart(fig, use_container_width=True)

            plotter = Plotter()
            fig = plotter.create_plot(df, "inlet_volume", "outlet_volume")

            st.subheader(f"📈 Volume Fluctuations")
            st.plotly_chart(fig, use_container_width=True)

            plotter = Plotter()
            fig = plotter.create_plot(df, "inlet_pressure", "outlet_pressure")

            st.subheader(f"📈 Pressure Fluctuations")
            st.plotly_chart(fig, use_container_width=True)

        except Exception as e:
            st.error(f"❌ Please check your input data/ date: {e}")

    else:
        st.info("📤 Please upload a CSV or Excel file to begin analysis.")


if __name__ == "__main__":
    show_csv_page()



import numpy as np
import pandas as pd
import collections
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

class SPRTLeakDetector:
    def __init__(self, window_size, global_window_factor=20, alpha=0.0005, beta=0.0005, decay_factor=0.999, data_buffer=None, filtered_buffer=None, mass1_buffer= None, mass2_buffer= None, calibration= False):
        self.window_size = window_size
        self.global_window_factor = global_window_factor
        self.alpha = alpha
        self.beta = beta
        self.decay_factor = decay_factor
        self.calibration= calibration

        self.A = np.log((1 - beta) / alpha) 
        self.B = np.log(beta / (1 - alpha))  

        if not data_buffer:
            self.mass1= collections.deque(maxlen=window_size)
        else:
            self.mass1= collections.deque(mass1_buffer, maxlen=window_size)
        if not data_buffer:
            self.mass2= collections.deque(maxlen=window_size)
        else:
            self.mass2= collections.deque(mass2_buffer, maxlen=window_size)

        if not data_buffer:
            self.data_buffer = collections.deque(maxlen=window_size)
        else:
            self.data_buffer = collections.deque(data_buffer, maxlen=window_size)
        
        if not filtered_buffer:
            self.filtered_buffer = collections.deque(maxlen=window_size*global_window_factor)  #list(np.zeros(1000))
        else:
            self.filtered_buffer = collections.deque(filtered_buffer, maxlen=window_size*global_window_factor)

        self.mass1_last = None
        self.mass2_last = None
        self.Z_n = 0
        self.previous_leak = False
        
        self.Z_values = []
        self.time_stamps = []
    
    def calculate_autocorrelation(self, data):
        try:
            if len(data) < 2:
                return 0
            
            data= data
            x_t = data[:-1]
            x_t1 = data[1:]

            if np.std(x_t)!=0 and np.std(x_t1)!=0:
                R_xx = np.corrcoef(x_t, x_t1)[0, 1]
                return (R_xx+1)/2
            else:
                return -1
        except:
            return -1
    
    def update_rolling_stats(self):
        global_window = self.window_size * self.global_window_factor

        filtered_array = np.array(self.filtered_buffer) if len(self.filtered_buffer) > 0 else np.array([0])
        mean_global = np.mean(filtered_array)
        std_global = np.std(filtered_array) 

        data_array = np.array(self.data_buffer) if len(self.data_buffer) > 0 else np.array([0])
        mean_window = np.mean(data_array)

        return mean_global, std_global, mean_window

    def process_new_data(self, mass1, mass2, timestamp, calibration_factor=0, delta_mass_previous=0):
        mass1 = mass1 if not pd.isna(mass1) else self.mass1_last
        mass2 = mass2 if not pd.isna(mass2) else self.mass2_last

        self.mass1.append(mass1)
        self.mass2.append(mass2)

        self.mass1_last, self.mass2_last = mass1, mass2
        
        delta_mass = mass1 - mass2
        
        self.data_buffer.append(delta_mass)
        
        if self.previous_leak==0:
            self.filtered_buffer.append(delta_mass)
        
        mu_0, sigma_0, mu_1 = self.update_rolling_stats()
        sigma_0 = sigma_0 if sigma_0 != 0 else 1e-5
        

        if len(self.filtered_buffer) >= self.window_size:
            R1 = self.calculate_autocorrelation(list(self.mass1)[-int(len(self.mass1)):])#-int(len(self.data_buffer)/10):])
            R2 = self.calculate_autocorrelation(list(self.mass2)[-int(len(self.mass2)):])
            if R1!=-1 and R2!=-1:
                R = (R1 + R2)/2
            elif R1!=-1:
                R=R1/2
            elif R2!=-1:
                R=R2/2
            else:
                R=0

            delta_mu = mu_1 - mu_0

            new_Z_n = (delta_mu / sigma_0**2/ (1+R)) * (delta_mass - R*delta_mass_previous -(1-R)*(mu_0 + 0.5 * delta_mu))
        else:
            new_Z_n = 0
        
        if self.previous_leak:
            self.Z_n = self.Z_n + new_Z_n
        else:
            if self.Z_n>0 and new_Z_n>0:
                self.Z_n = self.decay_factor * self.Z_n + new_Z_n
            else:
                self.Z_n = self.Z_n + new_Z_n
        if self.calibration==False:
            leak_detected = self.Z_n > self.A
        else:
            leak_detected= 0
        self.previous_leak = leak_detected

        self.Z_values.append(self.Z_n)
        self.time_stamps.append(timestamp)

        return self.Z_n


import streamlit as st
from datetime import datetime
from lds_pipelines.core.api_processor import get_starting_df, get_latest_df, check_alarm, login
from lds_pipelines.visualization.plotly_charts import Plotter
from streamlit_autorefresh import st_autorefresh
from datetime import datetime, timedelta
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

def show_api_page():
    st.title("🌐 Real-Time Leak Analysis")

    if 'api_running' not in st.session_state:
        st.session_state.api_running = False
        st.session_state.api_df = None
        st.session_state.api_interval = 5  # seconds
        st.session_state.leak_detected= False
        st.session_state.leak_time= None

    if not st.session_state.api_running:
        col1, col2 = st.columns(2)
        with col1:
            start_date = st.date_input("Select start date")
            start_time = st.time_input("Select start time")
        st.markdown("""---""")
        col3, col4, col5= st.columns(3)
        with col3:
            api_url= st.text_input("Enter OSI-PI API URL (optional)")
        with col4:
            api_username= st.text_input("Enter login ID (optional)")
        with col5:    
            api_password= st.text_input("Enter login password (optional)", type="password")
        st.markdown("""---""")
            
        with col2:
            pass
        start_dt = datetime.combine(start_date, start_time)
        start_iso = start_dt.isoformat()

        st.session_state.button_clicked = False

        if st.button("🚀 Start Analysis", disabled=st.session_state.button_clicked):
            if not st.session_state.api_running:
                login(api_url, api_username, api_password)
            st.session_state.button_clicked = True
            st.session_state.api_df = get_starting_df(start_iso)
            df=st.session_state.api_df
            st.session_state.api_running = True
            st.rerun()

    else:
        st_autorefresh(interval=int(st.session_state.api_interval * 1000), key="api_refresh")

        st.session_state.api_df = get_latest_df(st.session_state.api_df)
        df = st.session_state.api_df

        if st.session_state.leak_detected==False:            
            if check_alarm(df):
                leak_time= check_alarm(df)
                timestamp= leak_time- timedelta(hours=1)
                st.session_state.leak_time= timestamp
                st.session_state.leak_detected= True
                st.session_state.api_interval= 2
            
        else:
            st.warning(f'Leak has been detected at {st.session_state.leak_time}', icon="⚠️")

        plotter = Plotter()

        st.subheader("📈 Leak Likelihood")
        fig1 = plotter.create_plot(df, y1_column='min_zn')
        fig1.update_layout(uirevision="min_zn")
        st.plotly_chart(fig1, use_container_width=True)

        st.subheader("📈 Volume Fluctuations")
        fig2 = plotter.create_plot(df, y1_column='inlet_volume', y2_column='outlet_volume')
        fig2.update_layout(uirevision="volume_plot") 
        st.plotly_chart(fig2, use_container_width=True)

        st.subheader("📈 Pressure Fluctuations")
        fig3 = plotter.create_plot(df, y1_column='inlet_pressure', y2_column='outlet_pressure')
        fig3.update_layout(uirevision="pressure_plot")
        st.plotly_chart(fig3, use_container_width=True)

        if st.button("🛑 Stop"):
            st.session_state.api_running = False
            st.session_state.api_df = None
            st.rerun()


if __name__ == "__main__":
    pass




import streamlit as st
import subprocess
import sys
import os
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)


from lds_pipelines.dashboard.intro_page import show_intro_page  
from lds_pipelines.dashboard.csv_page import show_csv_page
from lds_pipelines.dashboard.api_page import show_api_page


def main():
    st.sidebar.title("Leak Detection System")
    
    pages = {
        "Introduction": show_intro_page,
        "Static Analysis": show_csv_page,
        "Real-time Analysis": show_api_page
    }
    
    selection = st.sidebar.radio("Go to", list(pages.keys()))
    
    pages[selection]()

    st.sidebar.markdown("---")
    st.sidebar.markdown('<div class="footer">LDS- \n Developed by Aman Sharma<br>© 2025</div>', unsafe_allow_html=True)
    st.sidebar.markdown("---")

def run():
    subprocess.run(["streamlit", "run", os.path.abspath(__file__)])
   

if __name__ == "__main__":
    st.set_page_config(
    page_title="Leak Detection Dashboard",
    page_icon="📊",
    layout="wide",
    initial_sidebar_state="expanded"
    )
    st.markdown("""
        <style>
        .sidebar .sidebar-content {
            display: flex;
            flex-direction: column;
            justify-content: space-between;
            height: 100%;
        }
        .footer {
            font-size: 0.9rem;
            color: gray;
            margin-top: 200;
        }
        </style>
    """, unsafe_allow_html=True)
    
    main()


import numpy as np
import pandas as pd
from lds_pipelines.core.sprt_detector import SPRTLeakDetector
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)


def calculate_zn_values(df):
    """Processes the input file and appends Z_n values for each row."""

    window_sizes = [300, 700, 1100] if len(df) > 300 else [10, 15]
    global_window_factor = 20
    
    detectors = {w: SPRTLeakDetector(window_size=w, global_window_factor=global_window_factor) for w in window_sizes}
    
    z_values_dict = {w: [] for w in window_sizes}
    min_Z_n = []

    for i in range(1, len(df), 1):
        intermediate_z_values = []
        for w in window_sizes:
            detectors[w].process_new_data(df.at[i, 'inlet_volume'], df.at[i, 'outlet_volume'], i, calibration_factor= 0, delta_mass_previous= (df.at[i-1, 'inlet_volume']-df.at[i-1, 'outlet_volume']))

        for w, detector in detectors.items():
            _, values = detector.time_stamps, detector.Z_values
            z_values_dict[w].append(values[-1])
            intermediate_z_values.append(values[-1]) 
        min_Z_n.append(min(intermediate_z_values))

    for w in window_sizes:
        df[f'zn_{w}'] = [0] + z_values_dict[w] 
    df['min_zn'] = [0] + min_Z_n 
    
    return df


if __name__ == "__main__":
    pass



"""
Introduction page for the Z_n analyzer dashboard.
"""
import streamlit as st
import pkg_resources


def show_intro_page():
    """Display the introduction page."""
    st.title("Leak Detection Analysis Dashboard")

    st.image(pkg_resources.resource_filename('lds_pipelines.dashboard', 'static_pipeline.jpg') , use_container_width=True)

    st.markdown("""
    ## 👋 Welcome to the Leak Detection Analysis Dashboard

    This dashboard helps you analyze **likelihood values of leakage** from your data — whether it's **static (CSV/Excel)** or **real-time (from OSI-PI server)**.

    ---

    ### 🚀 Key Features

    - **📁 CSV/Excel Analysis**: Upload historical data for static analysis using interactive charts.
    - **🌐 Real-time Analysis**: Connect live to an OSI-PI server and analyze streaming sensor data in real time.

    ---

    ### 🛠️ How to Use

    1. Select the desired analysis mode from the **sidebar**.
    2. For **static analysis**:
    - Upload a CSV or Excel file containing your sensor data.
    - View the interactive visualizations and Zn statistics.
    3. For **real-time analysis**:
    - Choose a **start date and time**.
    - The dashboard will begin streaming and visualizing the live Zn values.

    ---

    ## ℹ️ About Likelihood Values (Zn)

    The **Zn score** represents the system’s confidence in detecting disturbances or leaks.

    - Zn is computed using a modified **SPRT (Sequential Probability Ratio Test)** technique — designed for early and reliable detection.
    - When **Zn crosses the upper threshold**, the system is confident that a leak or disturbance **has likely occurred**.
    - If Zn stays **below the lower threshold**, the system is confident that **no disturbance has occurred so far**.

    ---
    
    """)
    


if __name__ == "__main__":
    show_intro_page()


import plotly.graph_objects as go
import pandas as pd

class Plotter:
    def __init__(self, title="Zn Values Over Time"):
        self.title = title

    def create_plot(self, df: pd.DataFrame, y1_column: str, y2_column: str = None, real_time= None):
        ...

        if not pd.api.types.is_datetime64_any_dtype(df['timestamps']):
            df['timestamps'] = pd.to_datetime(df['timestamps'], utc=True)

        if y2_column:
            mode = 'lines'
            line = dict(width=1)
        else:
            mode = 'lines'
            line = dict(width=3)

        fig = go.Figure()

        fig.add_trace(go.Scatter(
            x=df['timestamps'],
            y=df[y1_column],
            mode=mode,
            name=y1_column.replace('_', ' ').title(),
            line=line,
            yaxis='y1'
        ))

        if y2_column:
            fig.add_trace(go.Scatter(
                x=df['timestamps'],
                y=df[y2_column],
                mode=mode,
                name=y2_column.replace('_', ' ').title(),
                line=line,
                yaxis='y2'
            ))

        if not y2_column:
            fig.add_shape(
                type="line",
                x0=df['timestamps'].min(),
                x1=df['timestamps'].max(),
                y0=7.6,
                y1=7.6,
                line=dict(color="red", width=2, dash="dash"),
                yref="y",
                xref="x"
            )

            fig.add_shape(
                type="line",
                x0=df['timestamps'].min(),
                x1=df['timestamps'].max(),
                y0=-7.6,
                y1=-7.6,
                line=dict(color="green", width=2, dash="dash"),
                yref="y",
                xref="x"
            )

        total_days = max((df['timestamps'].max() - df['timestamps'].min()).days, 1)

        if real_time:
                layout = dict(
                height= 250,
                xaxis_title='Timestamp',
                yaxis=dict(
                    title=y1_column.replace('_', ' ').title(),
                    side='left'
                ),
                legend=dict(
                    orientation="h",
                    yanchor="bottom",
                    y=1.02,       
                    xanchor="right",
                    x=1,
                    font=dict(size=10),
                    bgcolor="rgba(0,0,0,0)"
                ),
                legend_title='',
                hovermode='x unified',
                xaxis=dict(
                    range=[df['timestamps'].min(), df['timestamps'].max()],
                    type="date",
                    autorange=True
                ),
            )
        else:
            layout = dict(
                title=self.title if not y2_column else "Fluctuations Over Time",
                xaxis_title='Timestamp',
                yaxis=dict(
                    title=y1_column.replace('_', ' ').title(),
                    side='left'
                ),
                legend=dict(
                    orientation="h",
                    yanchor="bottom",
                    y=1.02,       
                    xanchor="right",
                    x=1,
                    font=dict(size=10),
                    bgcolor="rgba(0,0,0,0)"
                ),
                legend_title='Legend',
                hovermode='x unified',
                xaxis=dict(
                    range=[df['timestamps'].min(), df['timestamps'].max()],
                    rangeselector=dict(
                        buttons=list([
                            dict(count=5, label="5m", step="minute", stepmode="backward"),
                            dict(count=10, label="10m", step="minute", stepmode="backward"),
                            dict(count=1, label="1h", step="hour", stepmode="backward"),
                            dict(count=1, label="1d", step="day", stepmode="backward"),
                            dict(count=total_days, label="All", step="day", stepmode="backward")
                        ])
                    ),
                    rangeslider=dict(visible=True,
                                    thickness=0.05,
                                    bgcolor='rgba(200, 200, 200, 0.2)', 
                                    bordercolor='rgba(0, 0, 0, 0.2)',
                                    borderwidth=1),
                    type="date",
                    autorange=False
                )
            )

        if y2_column:
            layout['yaxis2'] = dict(
                title=y2_column.replace('_', ' ').title(),
                overlaying='y',
                side='right'
            )

        fig.update_layout(**layout)

        return fig




if __name__=="__main__":
    file= "processed_output.csv"
    df= pd.read_csv(file)
    zn_plot= Plotter()
    fig= zn_plot.create_plot(df, 'outlet_volume', "inlet_volume")
    fig.show()


import pandas as pd
from datetime import datetime, timedelta
import requests
from requests.auth import HTTPBasicAuth
import urllib3
import json
import matplotlib.pyplot as plt
from lds_pipelines.core.sprt_detector import SPRTLeakDetector
from lds_pipelines.core.csv_processor import calculate_zn_values
import pkg_resources

urllib3.disable_warnings()

class MockAPIGenerator:
    def __init__(self, filepath):
        self.df = pd.read_csv(filepath)
        self.df['timestamps'] = pd.to_datetime(self.df['timestamps'])
        self.df = self.df.sort_values('timestamps').reset_index(drop=True)
        self.pointer = 0

    def get_starting_df(self, rows=10):
        self.pointer = rows
        return self.df.iloc[:self.pointer].copy()

    def get_next_df(self, step=1):
        if self.pointer >= len(self.df):
            return pd.DataFrame()

        next_pointer = min(self.pointer + step, len(self.df))
        result = self.df.iloc[self.pointer:next_pointer].copy()
        self.pointer = next_pointer
        return result

mockapi = MockAPIGenerator(pkg_resources.resource_filename('lds_pipelines.core', 'mock_api_data.csv'))#'./src/lds_pipeline/core/mock_api_data.csv')


def login(api_url, api_username, api_password):
    # Define PI Web API server details
    pi_web_api_url = api_url
    username = api_username
    password = api_password
    auth = HTTPBasicAuth(username, password)

    data_archive_name = "delews141odapri"
    pi_point_names = {
        "nqop": "PI-PS-MHANQONQP-DCS-PI2207",
        "nqof": "PI-PS-MHANQONQP-DCS-MUTOIL1FC2_NN3",
        "uranp": "PI-PS-URTTBYT-URN-MUT-21PT6101",
        "uranf": "PI-PS-URTTBYTURN-URN-FI6101"
    }

    web_ids = []
    web_id_data = [
        'F1DP8p7t-9NniEOgPfJJ19IjowaDgEAAREVMRVdTMTQxT0RBUFJJXFBJLVBTLU1IQU5RT05RUC1EQ1MtUEkyMjA3',
        'F1DP8p7t-9NniEOgPfJJ19IjowygEIAAREVMRVdTMTQxT0RBUFJJXFBJLVBTLU1IQU5RT05RUC1EQ1MtTVVUT0lMMUZDMl9OTjM',
        'F1DP8p7t-9NniEOgPfJJ19IjowKAIIAAREVMRVdTMTQxT0RBUFJJXFBJLVBTLVVSVFRCWVQtVVJOLU1VVC0yMVBUNjEwMQ',
        'F1DP8p7t-9NniEOgPfJJ19IjowfAIIAAREVMRVdTMTQxT0RBUFJJXFBJLVBTLVVSVFRCWVRVUk4tVVJOLUZJNjEwMQ',
    ]

now = datetime.now()

timestamps = []
inlet_pressure = []
inlet_volume = []
outlet_pressure = []
outlet_volume = []

def get_starting_df(start_time):
    # for i in range(len(web_ids)):
    #     interpolated_url = f"{pi_web_api_url}/streams/{web_ids[i]}/interpolated"

    #     params = {
    #     "startTime": f"{now}",
    #     "endTime": f"{start_time}",
    #     "interval": "1s",
    #     }

    #     response = requests.get(interpolated_url, auth=HTTPBasicAuth(username, password), params=params, verify=False)
    #     if response.status_code == 200:
    #         data = response.json()
    #         for item in data["Items"]:
    #             if i==0:
    #                 inlet_pressure.append(item["Value"])
    #                 timestamps.append(item["Timestamp"])
    #             if i==1:
    #                 inlet_volume.append(item["Value"])
    #             if i==2:
    #                 outlet_pressure.append(item["Value"])
    #             if i==3:
    #                 outlet_volume.append(item["Value"])
    #     else:
    #         print(f"Error: {response.status_code} - {response.reason}")

    # df = pd.DataFrame(list(zip(timestamps, inlet_pressure, inlet_volume, outlet_pressure, outlet_volume)),
    #                   columns=["timestamps", "inlet_pressure", "inlet_volume", "outlet_pressure", "outlet_volume"])

    df = mockapi.get_starting_df(rows=17000)
    df['timestamps'] = pd.to_datetime(df['timestamps'])
    df = df.sort_values('timestamps').reset_index(drop=True)
    df = calculate_zn_values(df)
    return df

def get_latest_df(df):
    last_timestamp = df['timestamps'].iloc[-1]


    current_timestamp = last_timestamp + timedelta(seconds=30)

    timestamps = df['timestamps'].tolist()
    inlet_volume = df['inlet_volume'].tolist()
    outlet_volume = df['outlet_volume'].tolist()
    new_inlet_pressure = df['inlet_pressure'].tolist()
    outlet_pressure = df['outlet_pressure'].tolist()

    window_sizes = [300, 700, 1100] if len(df) > 300 else [10, 15]
    global_window_factor = 20

    delta_mass = [inlet_volume[i] - outlet_volume[i] for i in range(len(inlet_volume))]

    detectors = {
        w: SPRTLeakDetector(
            window_size=w,
            global_window_factor=global_window_factor,
            data_buffer=delta_mass[-w:],
            filtered_buffer=delta_mass[-w * global_window_factor:],
            mass1_buffer=inlet_volume[-w:],
            mass2_buffer=outlet_volume[-w:]
        ) for w in window_sizes
    }


    # new_timestamps=[]
    # new_inlet_volume=[]
    # new_outlet_volume=[]
    # new_inlet_pressure=[]
    # new_outlet_pressure=[]


    # for i in range(len(web_ids)):
    #     interpolated_url = f"{pi_web_api_url}/streams/{web_ids[i]}/interpolated"

    #     params = {
    #     "startTime": f"{current_timestamp}",
    #     "endTime": f"{last_timestamp}",
    #     "interval": "5s",
    #     }

    #     response = requests.get(interpolated_url, auth=HTTPBasicAuth(username, password), params=params, verify=False)
    #     if response.status_code == 200:
    #         data = response.json()
    #         for item in data["Items"]:
    #             if i==0:
    #                 new_inlet_pressure.append(item["Value"])
    #                 new_timestamps.append(item["Timestamp"])
    #             if i==1:
    #                 new_inlet_volume.append(item["Value"])
    #             if i==2:
    #                 new_outlet_pressure.append(item["Value"])
    #             if i==3:
    #                 new_outlet_volume.append(item["Value"])
    #     else:
    #         print(f"Error: {response.status_code} - {response.reason}")

    # new_df= pd.DataFrame(list(zip(new_timestamps, new_inlet_volume, new_outlet_volume, new_inlet_pressure, new_outlet_pressure)) , columns=['timestamps', 'inlet_volume', 'outlet_volume', 'inlet_pressure', 'outlet_pressure'])
    # new_df = new_df.sort_values(by='timestamps', ascending=True).reset_index(drop=True)


    # for i in range(len(new_df)):
    #     timestamp = new_df['timestamps'].iloc[i]
    #     exists = timestamp in df['timestamps'].values
    #     if exists:
    #         pass
    #     else:
    #         latest_zn_values = {w: 0 for w in window_sizes}
    #         for w in detectors.keys():
    #             latest_zn_values[w] = detectors[w].process_new_data(new_row['inlet_volume'], new_row['outlet_volume'], new_row['timestamps'])
    #         new_row={   
    #                 "timestamps": new_df['timestamps'].iloc[i],
    #                 "inlet_volume": new_df['inlet_volume'].iloc[i],
    #                 "outlet_volume": new_df['outlet_volume'].iloc[i],
    #                 "inlet_pressure": new_df['inlet_pressure'].iloc[i],
    #                 "outlet_pressure": new_df['outlet_pressure'].iloc[i],
    #         }
    #         for w in window_sizes:
    #             new_row[f'zn_{w}']= latest_zn_values[w]
    #         df = pd.concat([df, pd.DataFrame([new_row])], ignore_index=True)
            



    result = mockapi.get_next_df()

    if result.empty:
        return df

    
    new_row = {}
    for col in result.columns:
        if col in ['timestamps', 'inlet_volume', 'outlet_volume', 'inlet_pressure', 'outlet_pressure']:
            new_row[col] = result[col].values[0]

    for w in window_sizes:
        new_row[f'zn_{w}'] = None
    new_row['min_zn'] = None

    df = pd.concat([df, pd.DataFrame([new_row])], ignore_index=True)

    latest_zn_values = {w: 0 for w in window_sizes}
    for w in detectors.keys():
        latest_zn_values[w] = detectors[w].process_new_data(new_row['inlet_volume'], new_row['outlet_volume'], new_row['timestamps'], delta_mass_previous=df['inlet_volume'].iloc[-2]-df['outlet_volume'].iloc[-2])

    for w in window_sizes:
        prev = df[f'zn_{w}'].iloc[-2] if len(df) > 1 else 0
        df.at[len(df) - 1, f'zn_{w}'] = prev + latest_zn_values[w]
    
    zn_vals=[]
    for w in window_sizes:
        zn_vals.append(df.loc[len(df) - 1, f'zn_{w}'])
    prev_min_zn = df['min_zn'].iloc[-2] if len(df) > 1 else 0
    df.at[len(df) - 1, 'min_zn'] = min(zn_vals)

    return df

def check_alarm(dataframe, column='min_zn', threshold=7.6):
    count = 0
    max_count = 0

    timestamps= pd.to_datetime(dataframe['timestamps'], utc=True)

    timestamp= None

    if len(dataframe)<300:
        max_counter=10
    else:
        max_counter=720


    for val in range(len(dataframe)):
        if dataframe[column].iloc[val] > threshold:
            count += 1
            max_count = max(max_count, count)
            timestamp= timestamps.iloc[val]
        else:
            count = 0  # reset streak
        if max_count>max_counter:
            return timestamp
        
    return False


if __name__=="__main__":
    pass